/*
    Kleurentester - Colour tester

    Copyright 2001, R.Harmsen

    Added Adsense Google advertisements May 2009
     5 Dec 2014: Added meta viewport
    10 May 2021: Removed Adsense code (already disabled since 2012)
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include "../cgitools/cgitools.h"

struct RedGreenBlue_s
{
   int Red;
   int Green;
   int Blue;
   char Name[20];
};
struct Colours_s
{
   struct RedGreenBlue_s Background;
   struct RedGreenBlue_s Text;
   struct RedGreenBlue_s Link;
   struct RedGreenBlue_s VisitedLink;
   struct RedGreenBlue_s ActiveLink;
};

static char *HexOption (int value);

static void showform (char *script, struct Colours_s *pColours);
static void ColourSelect (struct RedGreenBlue_s *pColType, char *Text);
static void htmlhead (struct Colours_s *pColours);
static void htmlfoot (void);
static void getparms (int len, struct Colours_s *pColours);
static int AnalColour (struct RedGreenBlue_s *pColType, char **V);
static int RandomColour (struct RedGreenBlue_s *pColType);

int main (void)
{
   int len;
   char *scriptname = "../cgi-bin/cores.cgi";

   struct Colours_s Colours = {
      { 0x10, 0x20, 0x30, "Background"},
      { 0xff, 0xff, 0xff, "Text"},
      { 0xff, 0x00, 0x00, "Link"},
      { 0x00, 0xff, 0x00, "VizLink"},
      { 0x50, 0xa0, 0xf0, "ActLink"} };

   if ((len = userinput()) == 0)
   {
      htmlhead(&Colours);
      showform(scriptname, &Colours);
      htmlfoot();
   }
   else
   {
      getparms(len, &Colours);
      htmlhead(&Colours);
      showform(scriptname, &Colours);
      htmlfoot();
   }
   return 0;
}

void showform (char *script, struct Colours_s *pColours)
{
   printf(
      "<FORM ACTION=\"%s\" METHOD=POST>\n"
      "<H1>Colour tester.</H1>\n"
      "<p><A TARGET = _top HREF=\"somewhere\">"
      "A link</A>\n"
      "&nbsp;&nbsp;"
      "<A TARGET = _top HREF=\"someware\">"
      "Another link</A>\n"
      "&nbsp;&nbsp;"
      "<A TARGET = _top HREF=\"somewhere else\">"
      "Yet another link</A></p>\n"
      , script);

   ColourSelect(&pColours->Background , "Background colour");
   ColourSelect(&pColours->Text       , "Text colour");
   ColourSelect(&pColours->Link       , "Link colour");
   ColourSelect(&pColours->VisitedLink, "Visited link");
   ColourSelect(&pColours->ActiveLink , "Active link");

   printf(
      "<INPUT TYPE=Submit Name=Redisplay VALUE=\"Redisplay\"> with new settings\n"
      "<INPUT TYPE=Submit Name=Randomise VALUE=\"Random\"> colours\n"
      "</FORM>\n\n");

   printf(
      "<p>Resulting Body statement:</p>\n"
      "<p>&lt;BODY BGCOLOR=\"#%02x%02x%02x\" "
      "TEXT=\"#%02x%02x%02x\"  "
      "LINK=\"#%02x%02x%02x\"  "
      "VLINK=\"#%02x%02x%02x\"  "
      "ALINK=\"#%02x%02x%02x\" &gt;</p>\n\n"
      ,
      pColours->Background.Red,
      pColours->Background.Green,
      pColours->Background.Blue,
      pColours->Text.Red,
      pColours->Text.Green,
      pColours->Text.Blue,
      pColours->Link.Red,
      pColours->Link.Green,
      pColours->Link.Blue,
      pColours->VisitedLink.Red,
      pColours->VisitedLink.Green,
      pColours->VisitedLink.Blue,
      pColours->ActiveLink.Red,
      pColours->ActiveLink.Green,
      pColours->ActiveLink.Blue
      );

   printf(
      "<p>Or better, in CSS (Cascading Style Sheets):</p>\n<p>\n"
      "<pre>body\n"
      "{                                   \n"
      "   background-color: #%02x%02x%02x; \n"
      "   color           : #%02x%02x%02x; \n"
      "}                                   \n"
      "a:link    {color: #%02x%02x%02x; }        \n"
      "a:visited {color: #%02x%02x%02x; }        \n"
      "a:active  {color: #%02x%02x%02x; }        \n"
      "</pre>\n"
      ,
      pColours->Background.Red,
      pColours->Background.Green,
      pColours->Background.Blue,
      pColours->Text.Red,
      pColours->Text.Green,
      pColours->Text.Blue,
      pColours->Link.Red,
      pColours->Link.Green,
      pColours->Link.Blue,
      pColours->VisitedLink.Red,
      pColours->VisitedLink.Green,
      pColours->VisitedLink.Blue,
      pColours->ActiveLink.Red,
      pColours->ActiveLink.Green,
      pColours->ActiveLink.Blue
      );

   printf("<p><a href=../>Site's front door</a></p>");
}

static void ColourSelect (struct RedGreenBlue_s *pColType, char *Text)
{
   printf("\n<p>\n");
   printf(
      "Red <SELECT NAME=%sR SIZE=1> %s </SELECT> ",
      pColType->Name, HexOption(pColType->Red));
   printf(
      "Green <SELECT NAME=%sG SIZE=1> %s </SELECT> ",
      pColType->Name, HexOption(pColType->Green));
   printf(
      "Blue <SELECT NAME=%sB SIZE=1> %s </SELECT> ",
      pColType->Name, HexOption(pColType->Blue));

   printf("%s</p>\n", Text);
}

void htmlhead (struct Colours_s *pColours)
{
   printf(
      "Content-type: text/html\n\n"
      "<HTML>\n"
      "<HEAD>\n"
      "<TITLE>Colour tester</TITLE>\n"
      "<meta name=viewport content=\"width=device-width, initial-scale=1\">\n"
      "</HEAD>\n"
      "<BODY BGCOLOR=\"#%02x%02x%02x\" "
      "TEXT=\"#%02x%02x%02x\"  "
      "LINK=\"#%02x%02x%02x\"  "
      "VLINK=\"#%02x%02x%02x\"  "
      "ALINK=\"#%02x%02x%02x\"  >\n",
      pColours->Background.Red,
      pColours->Background.Green,
      pColours->Background.Blue,
      pColours->Text.Red,
      pColours->Text.Green,
      pColours->Text.Blue,
      pColours->Link.Red,
      pColours->Link.Green,
      pColours->Link.Blue,
      pColours->VisitedLink.Red,
      pColours->VisitedLink.Green,
      pColours->VisitedLink.Blue,
      pColours->ActiveLink.Red,
      pColours->ActiveLink.Green,
      pColours->ActiveLink.Blue
      );

   fflush(stdout);
}

void htmlfoot (void)
{
   printf("<hr>\n");

   printf("</BODY></HTML>\n");
   fflush(stdout);
}

void getparms (int len, struct Colours_s *pColours)
{
   char *buf = calloc(len, 1);
   char **Vars, **V;
   int redisplay = 0;

   if (!buf || fread(buf, len, 1, stdin) != 1)
   {
      free(buf);
      return;
   }
   else
   {
      char *p;

      buf[len] = '\0';

      if ((p = strrchr(buf, '\n')) != NULL)
	 *p = '\0';
      if ((p = strrchr(buf, '\r')) != NULL)
	 *p = '\0';
   }

   {
      FILE *log = fopen("log", "a+t");

      if (log)
      {
         fprintf(log, "%.350s\n", buf);
         fclose(log);
      }
   }

   Vars = CgiConvVars(buf, NULL);

   for (V = Vars; V[0]; V += 2)
   {
      if (strcmp(V[0], "Redisplay") == 0)
      {
         redisplay = 1;
      }
   }
   if (redisplay)
   {
      for (V = Vars; V[0]; V += 2)
      {
         AnalColour(&pColours->Background , V);
         AnalColour(&pColours->Text       , V);
         AnalColour(&pColours->Link       , V);
         AnalColour(&pColours->VisitedLink, V);
         AnalColour(&pColours->ActiveLink , V);
      }
   }
   else
   {
      srand((unsigned)time(NULL));

      RandomColour(&pColours->Background);
      RandomColour(&pColours->Text);
      RandomColour(&pColours->Link);
      RandomColour(&pColours->VisitedLink);
      RandomColour(&pColours->ActiveLink);
   }

   free(buf);
   free(Vars);
}

static char *HexOption (int value)
{
   static char buf[16 * 2 * 25 + 20];
   char *p = buf;
   int i;

   for (i = 0x00; i <= 0xff; i += 0x08)
   {
      p += sprintf(p, "<Option %s>%02x</Option>\n",
         i == value ? "Selected" : "", i);
   }
   i = 0xff;
   p += sprintf(p, "<Option %s>%02x</Option>\n",
      i == value ? "Selected" : "", i);

   *p++ = '\n';
   *p++ = '\0';

   return buf;
}


static int AnalColour (struct RedGreenBlue_s *pColType, char **V)
{
   int l;

   l = strlen(pColType->Name);

   if (strncmp(V[0], pColType->Name, l) == 0)
   {
      int val = strtol(V[1], NULL, 16);
      switch (V[0][l])
      {
         case 'R':
            pColType->Red   = val;
            break;
         case 'G':
            pColType->Green = val;
            break;
         case 'B':
            pColType->Blue  = val;
            break;
      }
   }
   return 0;
}


static int RandomColour (struct RedGreenBlue_s *pColType)
{
   pColType->Red   = (rand() % 257 % 0xff) / 8 * 8;
   pColType->Green = (rand() % 257 % 0xff) / 8 * 8;
   pColType->Blue  = (rand() % 257 % 0xff) / 8 * 8;

   return 0;
}
